"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const solc = require("solc");
const fs = require("fs");
const fsx = require("fs-extra");
const pth = require("path");
const glob = require("fast-glob");
class Compiler {
    constructor(recipe) {
        this.output = {};
        this.input = {
            sources: Object.assign({}, (recipe ? recipe.sources : {})),
            language: recipe && recipe.language ? recipe.language : 'Solidity',
            settings: Object.assign({ evmVersion: 'byzantium', outputSelection: {
                    '*': {
                        '*': [
                            'abi', 'ast', 'legacyAST', 'devdoc', 'userdoc', 'metadata',
                            'ir', 'evm', 'evm.assembly', 'evm.legacyAssembly', 'evm.bytecode',
                            'evm.bytecode.object', 'evm.bytecode.opcodes', 'evm.bytecode.sourceMap',
                            'evm.bytecode.linkReferences', 'evm.deployedBytecode', 'evm.methodIdentifiers',
                            'evm.gasEstimates', 'ewasm', 'ewasm.wast', 'ewasm.wasm'
                        ],
                    },
                } }, (recipe ? recipe.settings : {})),
        };
        this.cwd = recipe && recipe.cwd ? recipe.cwd : process.cwd();
        this.reporter = recipe && recipe.reporter ? recipe.reporter : null;
    }
    source(...patterns) {
        const files = glob.sync(patterns, { cwd: this.cwd })
            .map((f) => f.toString());
        files.forEach((file) => {
            file = this.normalizePath(file);
            this.input.sources[file] = {
                content: fs.readFileSync(file).toString(),
            };
        });
        return files.length;
    }
    compile() {
        if (this.reporter) {
            this.reporter.onCompileStart(this);
        }
        const importer = (file) => {
            file = this.normalizePath(file);
            return {
                contents: fs.readFileSync(file).toString(),
            };
        };
        const input = JSON.stringify(this.input);
        this.output = JSON.parse(solc.compileStandardWrapper(input, importer));
        if (this.reporter) {
            this.reporter.onCompileEnd(this);
        }
        return !Array.isArray(this.output.errors);
    }
    save(dist) {
        if (this.reporter) {
            this.reporter.onSaveStart(this);
        }
        const target = pth.resolve(this.cwd, dist);
        fsx.ensureDirSync(target);
        let count = 0;
        Object.keys(this.output.contracts || {}).forEach((file) => {
            const sourcePath = this.normalizePath(file);
            const isModule = sourcePath.indexOf('./node_modules') === 0;
            if (isModule) {
                return;
            }
            const fileName = pth.basename(sourcePath);
            const contractName = fileName.split('.').slice(0, -1).join('.');
            const destPath = pth.join(target, `${contractName}.json`);
            const json = this.output.contracts[file];
            Object.keys(json).forEach((contract) => {
                if (json[contract].metadata) {
                    json[contract].metadata = JSON.parse(json[contract].metadata);
                }
            });
            const data = JSON.stringify(json, null, 2);
            fs.writeFileSync(destPath, data);
            count++;
        });
        if (this.reporter) {
            this.reporter.onSaveEnd(this);
        }
        return count;
    }
    clear() {
        this.output = {};
        return this;
    }
    normalizePath(path) {
        return path.indexOf('./') !== 0 ? `./node_modules/${path}` : path;
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=compiler.js.map